DROP DATABASE bibliotheque;
CREATE DATABASE bibliotheque;
USE bibliotheque;

CREATE TABLE Auteur (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    prenom VARCHAR(255) NOT NULL,
    nom VARCHAR(255) NOT NULL,
    anneeNaissance INT NOT NULL
);

CREATE TABLE Oeuvre (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    titre VARCHAR(255) NOT NULL,
    anneePremiereParution INT NOT NULL,
    auteurs_ids VARCHAR(255),
    mots_cles TEXT
);


CREATE TABLE Edition (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    nomEditeur VARCHAR(255) NOT NULL,
    anneeEdition INT NOT NULL,
    isbn VARCHAR(13) NOT NULL,
    oeuvre_id BIGINT,
    FOREIGN KEY (oeuvre_id) REFERENCES Oeuvre(id)
);

CREATE TABLE Exemplaire (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    edition_id BIGINT,
    emprunte BOOLEAN,
    FOREIGN KEY (edition_id) REFERENCES Edition(id)
);

CREATE TABLE CategorieUsager (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    nombreMaxEmprunts INT NOT NULL,
    dureeMaxEmprunt INT NOT NULL
);

CREATE TABLE Usager (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    prenom VARCHAR(255) NOT NULL,
    nom VARCHAR(255) NOT NULL,
    adresseEmail VARCHAR(255) NOT NULL,
    numeroIdentification VARCHAR(255) NOT NULL,
    surListeRouge BOOLEAN DEFAULT FALSE,
    categorie_usager_id BIGINT,
    Hashpassword VARCHAR(60) NOT NULL,
    salt VARCHAR(32) NOT NULL,
    FOREIGN KEY (categorie_usager_id) REFERENCES CategorieUsager(id)
);


CREATE TABLE Emprunt (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    dateEmprunt DATE NOT NULL,
    dateRetour DATE,
    usager_id BIGINT,
    exemplaire_id BIGINT,
    rendu BOOlEAN,
    FOREIGN KEY (usager_id) REFERENCES Usager(id),
    FOREIGN KEY (exemplaire_id) REFERENCES Exemplaire(id)
);

CREATE TABLE ListeRouge (
    id BIGINT AUTO_INCREMENT PRIMARY KEY,
    usager_id BIGINT,
    dateInterdiction DATE NOT NULL,
    FOREIGN KEY (usager_id) REFERENCES Usager(id)
);

-- Insertion dans la table CategorieUsager
INSERT INTO CategorieUsager (nombreMaxEmprunts, dureeMaxEmprunt) VALUES 
(3, 30), #Classique
(10, 60), #Premium
(100, 300); #Gestionnaire

-- Création du gestionnaire initial
INSERT INTO Usager (
    prenom, nom, adresseEmail, numeroIdentification, surListeRouge, categorie_usager_id, Hashpassword, salt
) VALUES (
    'Gestionnaire', 'Bibliothèque', 'gestion@bibliotheque.fr', 'GB031120240001', 0, 3, '$2a$12$GaxaG0BtN.DU8/lBf8eXmetg040V25r7Y0Jf3N/34DswpET6LkUDK', '$2a$12$GaxaG0BtN.DU8/lBf8eXme'
);

