package com.example.biblio;

import java.sql.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ExemplaireDataLoader {

    private DatabaseConnection databaseConnection;

    public ExemplaireDataLoader() {
        this.databaseConnection = new DatabaseConnection();
    }

    // Méthode pour récupérer toutes les œuvres et les lier aux auteurs, éditions, et exemplaires
    public List<Exemplaire> getAllExemplairesWithDetails() {
        List<Exemplaire> exemplaires = new ArrayList<>();

        String query = "SELECT o.*, ed.*, ex.* " +
                "FROM Oeuvre o " +
                "JOIN Edition ed ON o.id = ed.oeuvre_id " +
                "JOIN Exemplaire ex ON ed.id = ex.edition_id ";

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {

                // Créer l'objet Auteur
                List<Auteur> auteurs = new ArrayList<>();
                String[] auteurIds = resultSet.getString("o.auteurs_ids").split(",");
                for (String id : auteurIds) {
                    auteurs.add(Auteur.getAuteurById(Integer.parseInt(id)));
                }

                // Récupérer et transformer les mots-clés
                String motsClesString = resultSet.getString("mots_cles");
                List<String> motsCles = motsClesString != null ? Arrays.asList(motsClesString.split(",")) : new ArrayList<>();

                // Créer l'objet Oeuvre avec les mots-clés
                Oeuvre oeuvre = new Oeuvre(
                        resultSet.getString("titre"),
                        resultSet.getInt("anneePremiereParution"),
                        auteurs,
                        motsCles
                );
                oeuvre.setId(resultSet.getInt("o.id"));

                // Créer l'objet Edition
                Edition edition = new Edition(
                        resultSet.getString("nomEditeur"),
                        resultSet.getInt("anneeEdition"),
                        resultSet.getString("isbn"),
                        oeuvre // Associer l'œuvre à l'édition
                );
                edition.setId(resultSet.getInt("ed.id"));

                // Créer l'objet Exemplaire
                Exemplaire exemplaire = new Exemplaire(
                        edition,
                        resultSet.getBoolean("ex.emprunte") // Associer l'édition à l'exemplaire
                );
                exemplaire.setId(resultSet.getInt("ex.id"));

                // Ajouter l'exemplaire à la liste
                exemplaires.add(exemplaire);
            }

        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des données : " + e.getMessage());
        }

        return exemplaires;
    }
}