package com.example.biblio;

import java.sql.*;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Gestionnaire extends Usager {

    private DatabaseConnection databaseConnection;

    public Gestionnaire() throws Exception {
        super();
        this.databaseConnection = new DatabaseConnection();
    }

    // Méthode pour ajouter un exemplaire
    public void ajouterExemplaire(Exemplaire exemplaire) {
        String query = "INSERT INTO Exemplaire (edition_id, emprunte) VALUES (?, ?)";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, exemplaire.getEdition().getId());  // ID de l'édition
            preparedStatement.setBoolean(2, exemplaire.getEmprunte());
            preparedStatement.executeUpdate();
            System.out.println("Exemplaire ajouté avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de l'ajout de l'exemplaire : " + e.getMessage());
        }
    }

    // Méthode pour modifier un exemplaire
    public void modifierExemplaire(Exemplaire exemplaire) {
        String query = "UPDATE Exemplaire SET emprunte = ? WHERE id = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setBoolean(1, exemplaire.getEmprunte());
            preparedStatement.setInt(2, exemplaire.getId());  // ID de l'exemplaire
            preparedStatement.executeUpdate();
            System.out.println("Exemplaire modifié avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de la modification de l'exemplaire : " + e.getMessage());
        }
    }

    // Méthode pour supprimer un exemplaire
    public void supprimerExemplaire(Exemplaire exemplaire) {
        String query = "DELETE FROM Exemplaire WHERE id = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, exemplaire.getId());  // ID de l'exemplaire
            preparedStatement.executeUpdate();
            System.out.println("Exemplaire supprimé avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de la suppression de l'exemplaire : " + e.getMessage());
        }
    }

    // Méthode pour placer un usager en liste rouge
    public void placerUsagerEnListeRouge(Usager usager) {
        String query1 = "UPDATE Usager SET surListeRouge = ? WHERE id = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query1)) {
            preparedStatement.setInt(2, usager.getId());// ID de l'usager
            preparedStatement.setBoolean(1, !usager.getEnListeRouge());
            preparedStatement.executeUpdate();
            System.out.println("Usager placé/sorti de liste rouge avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors du placement de l'usager en liste rouge : " + e.getMessage());
        }
        if (!usager.getEnListeRouge()) {
            String query2 = "INSERT INTO ListeRouge (usager_id, dateInterdiction) VALUES (?,?)";

            try (Connection connection = databaseConnection.connect();
                 PreparedStatement preparedStatement = connection.prepareStatement(query2)) {
                preparedStatement.setInt(1, usager.getId());// ID de l'usager
                preparedStatement.setDate(2, Date.valueOf(LocalDate.now()));
                preparedStatement.executeUpdate();
                System.out.println("Création élément liste rouge avec succès.");
            } catch (SQLException e) {
                System.err.println("Erreur lors du placement de l'usager en liste rouge : " + e.getMessage());
            }

        } else {
            String query3 = "DELETE FROM ListeRouge WHERE usager_id = ?";
            try (Connection connection = databaseConnection.connect();
                 PreparedStatement preparedStatement = connection.prepareStatement(query3)) {
                preparedStatement.setInt(1, usager.getId());// ID de l'usager
                preparedStatement.executeUpdate();
                System.out.println("Suppression élément liste rouge avec succès.");
            } catch (SQLException e) {
                System.err.println("Erreur lors de la suppresion de la liste rouge : " + e.getMessage());
            }
        }
    }

    // Méthode pour récupérer tous les auteurs
    public List<Auteur> getAllAuteurs() {
        String query = "SELECT * FROM Auteur";
        List<Auteur> auteurs = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Auteur auteur = new Auteur();
                auteur.setId(resultSet.getInt("id"));
                auteur.setPrenom(resultSet.getString("prenom"));
                auteur.setNom(resultSet.getString("nom"));
                auteur.setAnneeNaissance(resultSet.getInt("anneeNaissance"));
                auteurs.add(auteur);
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des auteurs : " + e.getMessage());
        }
        return auteurs;
    }

    public List<Oeuvre> getAllLivres() {
        String query = "SELECT * FROM Oeuvre";
        List<Oeuvre> oeuvres = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Oeuvre oeuvre = new Oeuvre();
                oeuvre.setId(resultSet.getInt("id"));
                oeuvre.setTitre(resultSet.getString("titre"));
                oeuvre.setAnneePremiereParution(resultSet.getInt("anneePremiereParution"));

                // Récupérer les IDs des auteurs associés à l'oeuvre
                String auteursIds = resultSet.getString("auteurs_ids");
                List<Auteur> auteurs = getAuteursByIds(auteursIds, connection); // Appel à la méthode pour charger les auteurs
                oeuvre.setAuteurs(auteurs); // Assigner la liste des auteurs à l'oeuvre

                oeuvres.add(oeuvre);
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des livres : " + e.getMessage());
        }
        return oeuvres;
    }

    // Méthode pour récupérer une liste d'auteurs en fonction d'une chaîne d'IDs
    private List<Auteur> getAuteursByIds(String auteursIds, Connection connection) {
        List<Auteur> auteurs = new ArrayList<>();
        if (auteursIds != null && !auteursIds.isEmpty()) {
            String[] idsArray = auteursIds.split(",");
            String placeholders = String.join(",", Collections.nCopies(idsArray.length, "?"));

            String query = "SELECT * FROM Auteur WHERE id IN (" + placeholders + ")";

            try (PreparedStatement preparedStatement = connection.prepareStatement(query)) {
                for (int i = 0; i < idsArray.length; i++) {
                    preparedStatement.setInt(i + 1, Integer.parseInt(idsArray[i].trim()));
                }
                try (ResultSet resultSet = preparedStatement.executeQuery()) {
                    while (resultSet.next()) {
                        Auteur auteur = new Auteur();
                        auteur.setId(resultSet.getInt("id"));
                        auteur.setPrenom(resultSet.getString("prenom"));
                        auteur.setNom(resultSet.getString("nom"));
                        auteurs.add(auteur);
                    }
                }
            } catch (SQLException e) {
                System.err.println("Erreur lors de la récupération des auteurs : " + e.getMessage());
            }
        }
        return auteurs;
    }

    // Méthode pour récupérer toutes les éditions
    public List<Edition> getAllEditions() {
        String query = "SELECT * FROM Edition";
        List<Edition> editions = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Edition edition = new Edition();
                Oeuvre oeuvre = Oeuvre.getOeuvrebyID(resultSet.getInt("oeuvre_id"));
                edition.setId(resultSet.getInt("id"));
                edition.setOeuvre(oeuvre);
                edition.setNomEditeur(resultSet.getString("nomEditeur"));
                edition.setAnneeEdition(resultSet.getInt("anneeEdition"));
                edition.setIsbn(resultSet.getString("isbn"));

                editions.add(edition);
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des éditions : " + e.getMessage());
        }
        return editions;
    }

    // Méthode pour récupérer tous les exemplaires
    public List<Exemplaire> getAllExemplaires() {
        String query = "SELECT * FROM Exemplaire";
        List<Exemplaire> exemplaires = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Exemplaire exemplaire = new Exemplaire();
                exemplaire.setId(resultSet.getInt("id"));
                exemplaire.setEmprunte(resultSet.getBoolean("emprunte"));

                // Récupérer l'édition associée à l'exemplaire
                Edition edition = new Edition();
                edition.setId(resultSet.getInt("edition_id"));
                exemplaire.setEditionId(edition);

                exemplaires.add(exemplaire);
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des exemplaires : " + e.getMessage());
        }
        return exemplaires;
    }

    // Méthode pour ajouter un livre
    public void ajouterLivre(Oeuvre oeuvre) {
        String query = "INSERT INTO Oeuvre (titre, anneePremiereParution, auteurs_ids, mots_cles) VALUES (?, ?, ?, ?)";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, oeuvre.getTitre());
            preparedStatement.setInt(2, oeuvre.getAnneePremiereParution());
            preparedStatement.setString(3, oeuvre.getAuteursIds()); // IDs des auteurs
            String motsClesString = String.join(",", oeuvre.getKeywords());
            preparedStatement.setString(4, motsClesString);


            preparedStatement.executeUpdate();
            System.out.println("Livre ajouté avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de l'ajout du livre : " + e.getMessage());
        }
    }

    // Méthode pour ajouter une édition
    public void ajouterEdition(Edition edition) {
        String query = "INSERT INTO Edition (nomEditeur, anneeEdition, isbn, oeuvre_id) VALUES (?, ?, ?, ?)";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, edition.getNomEditeur());
            preparedStatement.setInt(2, edition.getAnneeEdition());
            preparedStatement.setString(3, edition.getIsbn());
            preparedStatement.setInt(4, edition.getOeuvre().getId());
            preparedStatement.executeUpdate();
            System.out.println("Édition ajoutée avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de l'ajout de l'édition : " + e.getMessage());
        }
    }

    // Méthode pour ajouter un auteur
    public void ajouterAuteur(Auteur auteur) {
        String query = "INSERT INTO Auteur (prenom, nom, anneeNaissance) VALUES (?, ?, ?)";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, auteur.getPrenom());
            preparedStatement.setString(2, auteur.getNom());
            preparedStatement.setInt(3, auteur.getAnneeNaissance());
            preparedStatement.executeUpdate();
            System.out.println("Auteur ajouté avec succès.");
        } catch (SQLException e) {
            System.err.println("Erreur lors de l'ajout de l'auteur : " + e.getMessage());
        }
    }

    // Méthode pour récupérer tous les usagers
    public List<Usager> getAllUsagers() {
        String query = "SELECT * FROM Usager";
        List<Usager> usagers = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Usager usager = new Usager(
                        resultSet.getString("prenom"),
                        resultSet.getString("nom"),
                        resultSet.getString("adresseEmail"),
                        resultSet.getString("numeroIdentification"),
                        resultSet.getInt("categorie_usager_id"),
                        resultSet.getString("Hashpassword"),
                        resultSet.getString("salt"),
                        resultSet.getBoolean("surListeRouge")
                );
                usagers.add(usager);
            }
        } catch (SQLException e) {
            System.out.println("Erreur lors de la récupération des usagers : " + e.getMessage());
        }
        return usagers;
    }
    public void changerCategorieUsager(Usager usager, int nouvelleCategorie) throws SQLException {
        String query = "UPDATE Usager SET categorie_usager_id = ? WHERE id = ?";
        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, nouvelleCategorie);
            preparedStatement.setInt(2, usager.getId());
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new SQLException("Impossible de mettre à jour la catégorie : " + e.getMessage());
        }
    }
}