package com.example.biblio;

import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.stage.Stage;
import javafx.scene.control.ListView;

import java.sql.SQLException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

public class GestionnaireController {

    @FXML
    private TextField searchBar; // Barre de recherche pour les exemplaires disponibles
    @FXML
    private TextField borrowedSearchBar; // Barre de recherche pour les exemplaires empruntés
    @FXML
    private TextField userSearchBar; // Barre de recherche pour les utilisateurs

    private FilteredList<Exemplaire> filteredNonEmpruntesList; // Liste filtrée pour les exemplaires non empruntés
    private FilteredList<Exemplaire> filteredEmpruntesList; // Liste filtrée pour les exemplaires empruntés
    private FilteredList<Usager> filteredUsagersList; // Liste filtrée pour les utilisateurs

    @FXML
    private TableView<Usager> tableUsagers;
    @FXML
    private TableView<Exemplaire> tableExemplairesNonEmpruntes;
    @FXML
    private TableView<Exemplaire> tableExemplairesEmpruntes;
    @FXML
    private TableView<Exemplaire> tableExemplairesUsagers;
    @FXML
    private TableView<Emprunt> tableHistoriqueEmprunts;

    @FXML
    private TableColumn<Usager, String> prenomUsagerColumn;
    @FXML
    private TableColumn<Usager, String> nomUsagerColumn;
    @FXML
    private TableColumn<Usager, String> emailUsagerColumn;
    @FXML
    private TableColumn<Usager, String> categorieColumn;

    @FXML
    private ComboBox<String> categorie;



    @FXML
    public TableColumn<Usager, String> enListeRouge;

    @FXML
    private TableColumn<Exemplaire, String> editionExemplaireColumn;
    @FXML
    private TableColumn<Exemplaire, Boolean> emprunteExemplaireColumn;

    @FXML
    private TableColumn<Exemplaire, String> titreColumn;
    @FXML
    private TableColumn<Exemplaire, String> auteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> dateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> editionColumn;
    @FXML
    private TableColumn<Exemplaire, String> keywordsColumn; // Colonne pour afficher les mots-clés

    @FXML
    private TextField keywordsField; // Champ pour saisir les mots-clés


    @FXML
    private TableColumn<Exemplaire, String> borrowedTitreColumn;
    @FXML
    private TableColumn<Exemplaire, String> borrowedAuteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> borrowedDateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> borrowedEditionColumn;

    @FXML
    private TableColumn<Exemplaire, String> borrowedReturnDateColumn;

    @FXML
    private TableColumn<Exemplaire, String> userBorrowedTitreColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedAuteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> userBorrowedDateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedEditionColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedReturnDateColumn;


    @FXML
    private ComboBox<Oeuvre> livreComboBox;
    @FXML
    private ComboBox<Edition> editionComboBox;

    @FXML
    private TextField nomAuteurField;
    @FXML
    private TextField prenomAuteurField;
    @FXML
    private TextField anneeNaissanceAuteurField;
    @FXML
    private TextField titreLivreField;
    @FXML
    private TextField anneePremiereParutionField;
    @FXML
    private TextField editeurField;
    @FXML
    private TextField anneeEditionField;
    @FXML
    private TextField isbnField;
    @FXML
    private TextField nombreExemplairesField;
    @FXML
    private ListView<Auteur> auteursListView;

    @FXML
    private TableColumn<Emprunt, String> historiqueTitreColumn;
    @FXML
    private TableColumn<Emprunt, String> historiqueUsagerColumn;
    @FXML
    private TableColumn<Emprunt, Date> historiqueDateEmpruntColumn;
    @FXML
    private TableColumn<Emprunt, Date> historiqueDateRetourColumn;
    @FXML
    private TableColumn<Emprunt, Boolean> historiqueRenduColumn;

    @FXML
    private TableView<ListeRouge> tableHistoriqueListeRouge;
    @FXML
    private TableColumn<ListeRouge, Date> historiqueListeRougeDateColumn;
    @FXML
    private TableColumn<ListeRouge, String> historiqueListeRougePrenomColumn;
    @FXML
    private TableColumn<ListeRouge, String> historiqueListeRougeNomColumn;
    @FXML
    private TableColumn<ListeRouge, String> historiqueListeRougeEmailColumn;

    private ObservableList<ListeRouge> historiqueListeRougeList = FXCollections.observableArrayList();


    private Gestionnaire gestionnaire;
    private ObservableList<Usager> usagersObservableList = FXCollections.observableArrayList();
    private ObservableList<Exemplaire> exemplairesNonEmpruntesList = FXCollections.observableArrayList();
    private ObservableList<Exemplaire> exemplairesEmpruntesList = FXCollections.observableArrayList();
    private ObservableList<Exemplaire> exemplairesUsagersList = FXCollections.observableArrayList();
    private ObservableList<Emprunt> historiqueEmpruntsList = FXCollections.observableArrayList();

    @FXML
    private Label usagerLabel;

    private Stage stage;
    private ExemplaireDataLoader exemplaireDataLoader;
    private ReservationService reservationService;
    private Usager usager;

    public GestionnaireController() throws Exception {
        gestionnaire = new Gestionnaire();
        exemplaireDataLoader = new ExemplaireDataLoader();
        reservationService = new ReservationService();
    }

    public void setStage(Stage stage) {
        this.stage = stage;
    }

    @FXML
    public void initialize() {
        try{
            usager = Session.getUsagerConnecte();
            if (usager != null) {
                // Déterminer la catégorie d'accès en fonction de categorieUsagerId
                String categorieAcces;
                switch (usager.getCategorieUsagerId()) {
                    case 1:
                        categorieAcces = "Classique";
                        break;
                    case 2:
                        categorieAcces = "Premium";
                        break;
                    case 3:
                        categorieAcces = "Gestionnaire";
                        break;
                    default:
                        categorieAcces = "Inconnue"; // Au cas où l'ID ne correspondrait pas
                }

                // Mettre à jour le label avec les informations de l'usager et la catégorie d'accès
                usagerLabel.setText("Connecté en tant que : " + usager.getPrenom() + " " + usager.getNom() +
                        "\nNuméro d'Identification : " + usager.getNumeroIdentification() +
                        "\nAccès : " + categorieAcces);
            }
            // Configurer les colonnes pour chaque propriété (onglet des non empruntés)
            titreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
            // Activer le mode de sélection multiple pour la ListView
            auteursListView.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
            auteurColumn.setCellValueFactory(cellData ->
                    new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
            dateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
            editionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));
            keywordsColumn.setCellValueFactory(cellData -> {
                List<String> keywords = cellData.getValue().getEdition().getOeuvre().getKeywords();
                return new SimpleStringProperty(String.join(", ", keywords));
            });

            // Configurer les colonnes pour les exemplaires empruntés (général)
            borrowedTitreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
            borrowedAuteurColumn.setCellValueFactory(cellData ->
                    new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
            borrowedDateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
            borrowedEditionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));



            // Configurer la colonne pour afficher la date de retour dans les exemplaires empruntés (général)
            borrowedReturnDateColumn.setCellValueFactory(cellData -> {
                Date dateRetour = reservationService.dateMaxEmprunt(cellData.getValue());
                return new SimpleStringProperty(dateRetour != null ? dateRetour.toString() : "Non défini");
            });

            // Configurer la colonne pour les utilisateurs
            prenomUsagerColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getPrenom()));
            nomUsagerColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getNom()));
            emailUsagerColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getAdresseEmail()));
            enListeRouge.setCellValueFactory(cellData -> new SimpleBooleanProperty(cellData.getValue().getEnListeRouge()).asString());
            categorieColumn.setCellValueFactory(cellData -> {
                int categorieId = cellData.getValue().getCategorieUsagerId();
                String categorieText;
                switch (categorieId) {
                    case 1:
                        categorieText = "Classique";
                        break;
                    case 2:
                        categorieText = "Premium";
                        break;
                    case 3:
                        categorieText = "Gestionnaire";
                        break;
                    default:
                        categorieText = "Inconnue";
                }
                return new SimpleStringProperty(categorieText);
            });
            // Appliquer une couleur de fond rouge aux emprunts non rendus à temps
            tableHistoriqueEmprunts.setRowFactory(tv -> new TableRow<Emprunt>() {
                @Override
                protected void updateItem(Emprunt emprunt, boolean empty) {
                    super.updateItem(emprunt, empty);

                    if (emprunt == null || empty) {
                        setStyle(""); // Réinitialiser le style pour les lignes vides
                    } else {
                        Date today = new Date();
                        Date dateRetour = emprunt.getDateRetour();
                        boolean rendu = emprunt.getRendu();

                        // Vérifier si la date de retour est dépassée et que l'emprunt n'est pas rendu
                        if (dateRetour != null && !rendu && dateRetour.before(today)) {
                            setStyle("-fx-background-color: red; -fx-text-fill: white;");
                        } else {
                            setStyle(""); // Réinitialiser le style si l'emprunt est rendu ou dans les temps
                        }
                    }
                }
            });

            historiqueTitreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getTitre()));
            historiqueUsagerColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getUsager().getAdresseEmail()));
            historiqueDateEmpruntColumn.setCellValueFactory(cellData -> new SimpleObjectProperty<>(cellData.getValue().getDateEmprunt()));
            historiqueDateRetourColumn.setCellValueFactory(cellData -> new SimpleObjectProperty<>(cellData.getValue().getDateRetour()));
            historiqueRenduColumn.setCellValueFactory(cellData -> new SimpleBooleanProperty(cellData.getValue().getRendu()));

            // Configuration des colonnes pour l'historique des placements en liste rouge
            historiqueListeRougeDateColumn.setCellValueFactory(cellData -> new SimpleObjectProperty<>(cellData.getValue().getDateListeRouge()));
            historiqueListeRougePrenomColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getUsager().getPrenom()));
            historiqueListeRougeNomColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getUsager().getNom()));
            historiqueListeRougeEmailColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getUsager().getAdresseEmail()));

            // Configurer les colonnes pour les exemplaires empruntés par l'usager
            userBorrowedTitreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
            userBorrowedAuteurColumn.setCellValueFactory(cellData ->
                    new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
            userBorrowedDateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
            userBorrowedEditionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));
            userBorrowedReturnDateColumn.setCellValueFactory(cellData -> {
                Date dateRetour = reservationService.dateMaxEmprunt(cellData.getValue());
                return new SimpleStringProperty(dateRetour != null ? dateRetour.toString() : "Non défini");
            });


        }catch (Exception e){
            e.printStackTrace();
        }

        chargerAuteurs();
        chargerLivres();
        chargerEditions();
        chargerUsagers();
        loadExemplaireData();
        loadHistoriqueEmprunts();
        try{
        loadHistoriqueListeRouge();}
        catch (Exception e){
            System.out.println(e.getMessage());
        }
        try{
        setupSearchFilter();}
        catch (Exception e){
            System.out.println(e.getMessage());
        }
    }

    private void setupSearchFilter() {
        // Créer une liste filtrée pour les exemplaires non empruntés
        filteredNonEmpruntesList = new FilteredList<>(exemplairesNonEmpruntesList, p -> true);
        tableExemplairesNonEmpruntes.setItems(filteredNonEmpruntesList);

        // Ajouter un écouteur à la barre de recherche
        searchBar.textProperty().addListener((observable, oldValue, newValue) -> {
            filteredNonEmpruntesList.setPredicate(exemplaire -> {
                if (newValue == null || newValue.isEmpty()) {
                    return true; // Affiche tous les exemplaires si la recherche est vide
                }

                // Filtrer par titre, auteur, date de parution ou éditeur
                String lowerCaseFilter = newValue.toLowerCase();
                Oeuvre oeuvre = exemplaire.getEdition().getOeuvre();
                boolean matchesKeywords = oeuvre.getKeywords().stream()
                        .anyMatch(keyword -> keyword.toLowerCase().contains(lowerCaseFilter));

                return exemplaire.getEdition().getOeuvre().getTitre().toLowerCase().contains(lowerCaseFilter)
                        || oeuvre.getNomsAuteurs().toLowerCase().contains(lowerCaseFilter)
                        || exemplaire.getEdition().getNomEditeur().toLowerCase().contains(lowerCaseFilter)
                        || matchesKeywords;
            });
        });

        // Configurer la recherche pour les exemplaires empruntés
        filteredEmpruntesList = new FilteredList<>(exemplairesEmpruntesList, p -> true);
        tableExemplairesEmpruntes.setItems(filteredEmpruntesList);
        borrowedSearchBar.textProperty().addListener((observable, oldValue, newValue) -> {
            filteredEmpruntesList.setPredicate(exemplaire -> {
                if (newValue == null || newValue.isEmpty()) {
                    return true;
                }
                String lowerCaseFilter = newValue.toLowerCase();
                return exemplaire.getEdition().getOeuvre().getTitre().toLowerCase().contains(lowerCaseFilter)
                        || exemplaire.getEdition().getOeuvre().getNomsAuteurs().toLowerCase().contains(lowerCaseFilter)
                        || exemplaire.getEdition().getNomEditeur().toLowerCase().contains(lowerCaseFilter);
            });
        });

        // Configurer la recherche pour les utilisateurs
        filteredUsagersList = new FilteredList<>(usagersObservableList, p -> true);
        tableUsagers.setItems(filteredUsagersList);
        userSearchBar.textProperty().addListener((observable, oldValue, newValue) -> {
            filteredUsagersList.setPredicate(usager -> {
                if (newValue == null || newValue.isEmpty()) {
                    return true;
                }
                String lowerCaseFilter = newValue.toLowerCase();
                return usager.getPrenom().toLowerCase().contains(lowerCaseFilter)
                        || usager.getNom().toLowerCase().contains(lowerCaseFilter)
                        || usager.getAdresseEmail().toLowerCase().contains(lowerCaseFilter);
            });
        });
    }

    private void loadExemplaireData() {
        List<Exemplaire> exemplaires = exemplaireDataLoader.getAllExemplairesWithDetails();
        exemplairesNonEmpruntesList.clear();
        exemplairesEmpruntesList.clear();
        exemplairesUsagersList.clear();


        // Filtrer les exemplaires non empruntés et empruntés
        for (Exemplaire exemplaire : exemplaires) {
            if (exemplaire.getEmprunte()) {
                if (reservationService.isExemplaireBorrowedByUser(exemplaire, usager)) {
                    exemplairesUsagersList.add(exemplaire);
                }
                exemplairesEmpruntesList.add(exemplaire);
            } else {
                exemplairesNonEmpruntesList.add(exemplaire);
            }
        }

        // Assigner les données aux TableView
        try{
        tableExemplairesNonEmpruntes.setItems(exemplairesNonEmpruntesList);
        tableExemplairesUsagers.setItems(exemplairesUsagersList);}
        catch (Exception e){
            System.out.println(e.getMessage());
        }

        tableExemplairesEmpruntes.setItems(exemplairesEmpruntesList);
    }

    private void chargerAuteurs() {
        ObservableList<Auteur> auteurs = FXCollections.observableArrayList(gestionnaire.getAllAuteurs());
        auteursListView.setItems(auteurs);
    }

    private void chargerLivres() {
        livreComboBox.setItems(FXCollections.observableArrayList(gestionnaire.getAllLivres()));
    }

    private void chargerEditions() {
        editionComboBox.setItems(FXCollections.observableArrayList(gestionnaire.getAllEditions()));
    }

    private void chargerUsagers() {
        try {
            List<Usager> usagers = gestionnaire.getAllUsagers();
            usagersObservableList.setAll(usagers);
            tableUsagers.setItems(usagersObservableList);
        } catch (Exception e) {
            afficherMessage("Erreur", "Erreur lors du chargement des usagers : " + e.getMessage());
        }
    }

    private void loadHistoriqueEmprunts() {
        List<Emprunt> emprunts = Emprunt.getEmprunts();
        historiqueEmpruntsList.setAll(emprunts);
        tableHistoriqueEmprunts.setItems(historiqueEmpruntsList);
    }

    private void loadHistoriqueListeRouge() {
        historiqueListeRougeList.setAll(ListeRouge.getHistoriqueListeRouge());
        tableHistoriqueListeRouge.setItems(historiqueListeRougeList);
    }

    @FXML
    private void ajouterAuteur() {
        String nom = nomAuteurField.getText();
        String prenom = prenomAuteurField.getText();
        int anneeNaissance = Integer.parseInt(anneeNaissanceAuteurField.getText());
        Auteur auteur = new Auteur(prenom, nom, anneeNaissance);

        gestionnaire.ajouterAuteur(auteur);
        afficherMessage("Succès", "Auteur ajouté avec succès.");
        chargerAuteurs();
        setupSearchFilter();
    }

    @FXML
    private void ajouterLivre() {
        String titre = titreLivreField.getText();
        Auteur auteurSelectionne = auteursListView.getSelectionModel().getSelectedItem();
        int anneePremiereParution = Integer.parseInt(anneePremiereParutionField.getText());

        ObservableList<Auteur> auteursSelectionnes = auteursListView.getSelectionModel().getSelectedItems();
        if (auteursSelectionnes.isEmpty()) {
            afficherMessage("Erreur", "Veuillez sélectionner au moins un auteur.");
            return;
        }

        List<Auteur> auteurs = new ArrayList<>(auteursSelectionnes);

        // Récupérer les mots-clés de la zone de texte, séparés par des virgules
        String keywordsText = keywordsField.getText();
        List<String> keywords = Arrays.asList(keywordsText.split(","));

        // Vérifier le nombre de mots-clés (entre 1 et 5)
        if (keywords.isEmpty() || keywords.size() > 5) {
            afficherMessage("Erreur", "Veuillez entrer entre 1 et 5 mots-clés.");
            return;
        }


        if (auteurSelectionne != null) {
            Oeuvre oeuvre = new Oeuvre(titre, anneePremiereParution, auteurs, keywords);
            gestionnaire.ajouterLivre(oeuvre);
            afficherMessage("Succès", "Livre ajouté avec succès.");
            chargerLivres();
            setupSearchFilter();
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un auteur.");
        }
    }

    @FXML
    private void emprunterLivre() {
        Exemplaire exemplaireSelectionne = tableExemplairesNonEmpruntes.getSelectionModel().getSelectedItem();
        Usager usager = Session.getUsagerConnecte();

        if (exemplaireSelectionne != null && usager != null) {
            try {
                reservationService.creerReservation(usager, exemplaireSelectionne); // Créer une réservation
            } catch (IllegalStateException e) {
                // Afficher un message d'avertissement si l'utilisateur a atteint le nombre maximal d'emprunts
                afficherMessage("Erreur", e.getMessage());
                return;
            }
            exemplaireSelectionne.setEmprunte(true);  // Marquer l'exemplaire comme emprunté

            // Vider les listes existantes
            exemplairesNonEmpruntesList.clear();
            exemplairesEmpruntesList.clear();
            exemplairesUsagersList.clear();

            // Vider les données dans les TableView
            tableExemplairesNonEmpruntes.getItems().clear();
            tableExemplairesEmpruntes.getItems().clear();
            tableExemplairesUsagers.getItems().clear();

            // Recharger les nouvelles données depuis la base
            loadExemplaireData();
            tableExemplairesNonEmpruntes.refresh();
            tableExemplairesEmpruntes.refresh();
            tableExemplairesUsagers.refresh();

            setupSearchFilter();

            afficherMessage("Succès", "Livre emprunté avec succès !");
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un livre à emprunter.");
        }
    }

    @FXML
    private void retournerLivre() {
        Exemplaire exemplaireSelectionne = tableExemplairesEmpruntes.getSelectionModel().getSelectedItem();
        Usager usager = getSelectedUsager();

        if (exemplaireSelectionne != null && usager != null) {
            reservationService.annulerReservation(usager, exemplaireSelectionne);  // Annuler la réservation
            exemplaireSelectionne.setEmprunte(false);  // Marquer l'exemplaire comme non emprunté

            // Vider les listes existantes
            exemplairesNonEmpruntesList.clear();
            exemplairesEmpruntesList.clear();
            exemplairesUsagersList.clear();

            // Vider les données dans les TableView
            tableExemplairesNonEmpruntes.getItems().clear();
            tableExemplairesEmpruntes.getItems().clear();
            tableExemplairesUsagers.getItems().clear();

            // Recharger les nouvelles données depuis la base
            loadExemplaireData();
            tableExemplairesNonEmpruntes.refresh();
            tableExemplairesEmpruntes.refresh();
            tableExemplairesUsagers.refresh();

            setupSearchFilter();

            afficherMessage("Succès", "Livre retourné avec succès !");
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un livre à retourner.");
        }
    }

    @FXML
    private void ajouterEdition() {
        Oeuvre livreSelectionne = livreComboBox.getSelectionModel().getSelectedItem();
        if (livreSelectionne != null) {
            String nomEditeur = editeurField.getText();
            int anneeEdition = Integer.parseInt(anneeEditionField.getText());
            String isbn = isbnField.getText();
            Edition edition = new Edition(nomEditeur, anneeEdition, isbn, livreSelectionne);

            gestionnaire.ajouterEdition(edition);
            afficherMessage("Succès", "Édition ajoutée avec succès.");
            chargerEditions();
            setupSearchFilter();
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un livre.");
        }
    }

    @FXML
    private void ajouterExemplaires() {
        Edition editionSelectionnee = editionComboBox.getSelectionModel().getSelectedItem();
        if (editionSelectionnee != null) {
            int nombreExemplaires = Integer.parseInt(nombreExemplairesField.getText());

            for (int i = 0; i < nombreExemplaires; i++) {
                Exemplaire exemplaire = new Exemplaire(editionSelectionnee, false);
                gestionnaire.ajouterExemplaire(exemplaire);
            }
            afficherMessage("Succès", "Exemplaires ajoutés avec succès.");
            loadExemplaireData();
            setupSearchFilter();
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner une édition.");
        }
    }

    @FXML
    private void placerUsagerEnListeRouge() throws SQLException { //Mettre ou Sortir de la Liste Rouge
        Usager usagerSelectionne = getSelectedUsager();
        //Placer en liste rouge

        if (usagerSelectionne != null) {
            try {
                gestionnaire.placerUsagerEnListeRouge(usagerSelectionne);

            }catch (Exception e) {
                afficherMessage("Erreur", e.getMessage());
            }
            afficherMessage("Succès", "Modification de la liste rouge effectuée avec succès.");
            chargerUsagers();
            loadHistoriqueListeRouge();
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un usager.");
        }
    }

    private Usager getSelectedUsager() {
        return tableUsagers.getSelectionModel().getSelectedItem();
    }

    private void afficherMessage(String titre, String contenu) {
        Alert alert = new Alert(Alert.AlertType.INFORMATION);
        alert.setTitle(titre);
        alert.setContentText(contenu);
        alert.showAndWait();
    }

    @FXML
    private void logout() {
        Session.setUsagerConnecte(null);
        Stage stage = (Stage) nomAuteurField.getScene().getWindow();
        stage.close();
        HelloApplication.showLoginWindow();
    }

    public void sortirDeListeRouge() {
        Usager usagerSelectionne = getSelectedUsagerListeRouge();
        System.out.println(usagerSelectionne);
        // Placer/Sortir en liste rouge

        if (usagerSelectionne != null) {
            try {
                gestionnaire.placerUsagerEnListeRouge(usagerSelectionne);

            }catch (Exception e) {
                afficherMessage("Erreur", e.getMessage());
            }
            afficherMessage("Succès", "Modification de la liste rouge effectuée avec succès.");
            chargerUsagers();
            loadHistoriqueListeRouge();
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un usager.");
        }
    }

    private Usager getSelectedUsagerListeRouge() {
        return tableHistoriqueListeRouge.getSelectionModel().getSelectedItem().getUsager();
    }
    @FXML
    public void changerCategorie() {
        Usager usagerSelectionne = tableUsagers.getSelectionModel().getSelectedItem();
        int nouvelleCategorie = categorie.getSelectionModel().getSelectedIndex() + 1;
        String nomNouvelleCategorie = categorie.getSelectionModel().getSelectedItem();

        if (usagerSelectionne == null) {
            afficherMessage("Erreur", "Veuillez sélectionner un utilisateur.");
            return;
        }

        if (nomNouvelleCategorie == null) {
            afficherMessage("Erreur", "Veuillez sélectionner une catégorie.");
            return;
        }

        try {
            // Appel de la méthode du gestionnaire pour changer la catégorie
            gestionnaire.changerCategorieUsager(usagerSelectionne, nouvelleCategorie);
            System.out.println(nouvelleCategorie);

            // Mettre à jour la table des utilisateurs pour refléter le changement
            chargerUsagers();
            tableUsagers.refresh();

            afficherMessage("Succès", "Catégorie modifiée avec succès.");
        } catch (Exception e) {
            afficherMessage("Erreur", "Erreur lors du changement de catégorie : " + e.getMessage());
        }
    }
}