package com.example.biblio;

import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.stage.Stage;

import java.sql.SQLException;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.Date;
import java.util.List;

public class HelloController {


    @FXML
    private TextField searchBar; // Ajout de la barre de recherche
    private FilteredList<Exemplaire> filteredNonEmpruntesList; // Liste filtrée pour les exemplaires non empruntés


    @FXML
    private TableView<Exemplaire> nonBorrowedTableView; // Pour les exemplaires non empruntés
    @FXML
    private TableView<Exemplaire> borrowedTableView; // Pour tous les exemplaires empruntés
    @FXML
    private TableView<Exemplaire> currentUserBorrowedTableView; // Pour les exemplaires empruntés par l'usager actuel

    @FXML
    private TableColumn<Exemplaire, String> titreColumn;
    @FXML
    private TableColumn<Exemplaire, String> auteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> dateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> editionColumn;
    @FXML
    private TableColumn<Exemplaire, String> keywordsColumn; // Colonne pour afficher les mots-clés

    @FXML
    private TableColumn<Exemplaire, String> borrowedTitreColumn;
    @FXML
    private TableColumn<Exemplaire, String> borrowedAuteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> borrowedDateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> borrowedEditionColumn;

    @FXML
    private TableColumn<Exemplaire, String> userBorrowedTitreColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedAuteurColumn;
    @FXML
    private TableColumn<Exemplaire, Integer> userBorrowedDateParutionColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedEditionColumn;

    @FXML
    private TableColumn<Exemplaire, String> borrowedReturnDateColumn;
    @FXML
    private TableColumn<Exemplaire, String> userBorrowedReturnDateColumn;

    @FXML
    private Button btnLogout1, btnLogout2, btnLogout3;
    @FXML
    private Button btnBorrowBook;
    @FXML
    private Button btnReturnBook;

    @FXML
    private Label usagerLabel1, usagerLabel2, usagerLabel3;


    private ExemplaireDataLoader exemplaireDataLoader;
    private ReservationService reservationService;

    private ObservableList<Exemplaire> exemplaireNonEmpruntesList = FXCollections.observableArrayList();
    private ObservableList<Exemplaire> exemplaireEmpruntesList = FXCollections.observableArrayList();
    private ObservableList<Exemplaire> exemplaireEmpruntesParUsagerList = FXCollections.observableArrayList();


    public HelloController() {
    }

    @FXML
    public void initialize() {
        exemplaireDataLoader = new ExemplaireDataLoader();
        reservationService = new ReservationService();

        Usager usager = Session.getUsagerConnecte();
        if (usager != null) {
            usager = Session.getUsagerConnecte();
            if (usager != null) {
                // Déterminer la catégorie d'accès en fonction de categorieUsagerId
                String categorieAcces;
                switch (usager.getCategorieUsagerId()) {
                    case 1:
                        categorieAcces = "Classique";
                        break;
                    case 2:
                        categorieAcces = "Premium";
                        break;
                    case 3:
                        categorieAcces = "Gestionnaire";
                        break;
                    default:
                        categorieAcces = "Inconnue"; // Au cas où l'ID ne correspondrait pas
                }

                // Mettre à jour le label avec les informations de l'usager et la catégorie d'accès
                String texte = "Connecté en tant que : " + usager.getPrenom() + " " + usager.getNom() +
                        ", Numéro d'Identification : " + usager.getNumeroIdentification() +
                        ", Accès : " + categorieAcces;
                usagerLabel1.setText(texte);
                usagerLabel2.setText(texte);
                usagerLabel3.setText(texte);
            }
        }

        // Configurer les colonnes pour chaque propriété (onglet des non empruntés)
        titreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
        auteurColumn.setCellValueFactory(cellData ->
                new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
        dateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
        editionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));
        keywordsColumn.setCellValueFactory(cellData -> {
            List<String> keywords = cellData.getValue().getEdition().getOeuvre().getKeywords();
            return new SimpleStringProperty(String.join(", ", keywords));
        });

        // Configurer les colonnes pour les exemplaires empruntés (général)
        borrowedTitreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
        borrowedAuteurColumn.setCellValueFactory(cellData ->
                new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
        borrowedDateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
        borrowedEditionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));

        // Configurer la colonne pour afficher la date de retour dans les exemplaires empruntés (général)
        borrowedReturnDateColumn.setCellValueFactory(cellData -> {
            Date dateRetour = reservationService.dateMaxEmprunt(cellData.getValue());
            return new SimpleStringProperty(dateRetour != null ? dateRetour.toString() : "Non défini");
        });

        // Configurer les colonnes pour les exemplaires empruntés par l'usager
        userBorrowedTitreColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getTitre()));
        userBorrowedAuteurColumn.setCellValueFactory(cellData ->
                new SimpleStringProperty(cellData.getValue().getEdition().getOeuvre().getNomsAuteurs()));
        userBorrowedDateParutionColumn.setCellValueFactory(cellData -> new SimpleIntegerProperty(cellData.getValue().getEdition().getOeuvre().getAnneePremiereParution()).asObject());
        userBorrowedEditionColumn.setCellValueFactory(cellData -> new SimpleStringProperty(cellData.getValue().getEdition().getNomEditeur()));

        // Configurer la colonne pour afficher la date de retour dans les exemplaires empruntés par l'utilisateur
        userBorrowedReturnDateColumn.setCellValueFactory(cellData -> {
            Date dateRetour = reservationService.dateMaxEmprunt(cellData.getValue());
            return new SimpleStringProperty(dateRetour != null ? dateRetour.toString() : "Non défini");
        });

        // Charger les données
        loadExemplaireData();

        // Configurer le filtre de recherche
        setupSearchFilter();

        // Vérifier si l'Usager est dans la liste rouge
        if (usager.getEnListeRouge()) {
            afficherMessage("Attention !", "Vous avez été placé en liste rouge car vous n'avez pas respecté les conditions de notre bibliothèque. " +
                    "Pour pouvoir emprunter un nouveau livre vous devez rendre tous vos livres à un gestionnaire de notre bibliothèque et lui demander de vous sortir de la liste.");
            disableButton();
        }
        LocalDate today = LocalDate.now();
        List<Emprunt> emprunts = Emprunt.getEmprunts();
        for (Emprunt emprunt : emprunts) {
            if (emprunt.getUsager().getId() == usager.getId()) {
                try {
                    Date dateRetour = emprunt.getDateRetour();  // Date de retour d'emprunt
                    // Convertir Date en LocalDate
                    LocalDate dateRetourLocal = ((java.sql.Date) dateRetour).toLocalDate();
                    // Comparer si today est après la date de retour
                    if (today.isAfter(dateRetourLocal) && !emprunt.getRendu()) {
                        afficherMessage("Attention", "Vous êtes en retard pour rendre le livre :" + emprunt.getTitre() +
                                " Veuillez le rendre au plus vite si vous ne voulez pas être placé en liste rouge.");
                    }
                }catch (Exception e) {
                    System.out.println(e.getMessage());
                }
            }
        }
    }

    public void disableButton() {
        btnBorrowBook.setDisable(true); // Désactive le bouton
    }

    private void setupSearchFilter() {
        // Créer une liste filtrée pour les exemplaires non empruntés
        filteredNonEmpruntesList = new FilteredList<>(exemplaireNonEmpruntesList, p -> true);
        nonBorrowedTableView.setItems(filteredNonEmpruntesList);

        // Ajouter un écouteur à la barre de recherche
        searchBar.textProperty().addListener((observable, oldValue, newValue) -> {
            filteredNonEmpruntesList.setPredicate(exemplaire -> {
                if (newValue == null || newValue.isEmpty()) {
                    return true; // Affiche tous les exemplaires si la recherche est vide
                }

                // Filtrer par titre, auteur, date de parution, éditeur ou mots-clés
                String lowerCaseFilter = newValue.toLowerCase();
                Oeuvre oeuvre = exemplaire.getEdition().getOeuvre();

                boolean matchesKeywords = oeuvre.getKeywords().stream()
                        .anyMatch(keyword -> keyword.toLowerCase().contains(lowerCaseFilter));

                return oeuvre.getTitre().toLowerCase().contains(lowerCaseFilter)
                        || oeuvre.getNomsAuteurs().toLowerCase().contains(lowerCaseFilter)
                        || oeuvre.getKeywords().toString().toLowerCase().contains(lowerCaseFilter) // Recherche dans les mots-clés
                        || exemplaire.getEdition().getNomEditeur().toLowerCase().contains(lowerCaseFilter)
                        || matchesKeywords;
            });
        });
    }

    // Méthode pour emprunter un livre
    @FXML
    private void emprunterLivre() {
        Exemplaire exemplaireSelectionne = nonBorrowedTableView.getSelectionModel().getSelectedItem();
        Usager usager = Session.getUsagerConnecte();

        if (exemplaireSelectionne != null && usager != null) {
            try{
            reservationService.creerReservation(usager, exemplaireSelectionne);}// Créer une réservation
            catch (IllegalStateException e) {
                // Afficher un message d'avertissement si l'utilisateur a atteint le nombre maximal d'emprunts
                afficherMessage("Erreur", e.getMessage());
                return;
            }
            exemplaireSelectionne.setEmprunte(true);  // Marquer l'exemplaire comme emprunté

            // Mettre à jour les listes

            // Vider les listes existantes
            exemplaireNonEmpruntesList.clear();
            exemplaireEmpruntesList.clear();
            exemplaireEmpruntesParUsagerList.clear();

            // Vider les données dans les TableView
            nonBorrowedTableView.getItems().clear();
            borrowedTableView.getItems().clear();
            currentUserBorrowedTableView.getItems().clear();

            // Recharger les nouvelles données depuis la base
            loadExemplaireData();
            nonBorrowedTableView.refresh();
            borrowedTableView.refresh();
            currentUserBorrowedTableView.refresh();

            setupSearchFilter();

            afficherMessage("Succès", "Livre emprunté avec succès !");
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un livre à emprunter.");
        }
    }

    // Méthode pour retourner un livre
    @FXML
    private void retournerLivre() {
        Exemplaire exemplaireSelectionne = currentUserBorrowedTableView.getSelectionModel().getSelectedItem();
        Usager usager = Session.getUsagerConnecte();

        if (exemplaireSelectionne != null && usager != null) {
            reservationService.annulerReservation(usager, exemplaireSelectionne);  // Annuler la réservation
            exemplaireSelectionne.setEmprunte(false);// Marquer l'exemplaire comme non emprunté

            // Vider les listes existantes
            exemplaireNonEmpruntesList.clear();
            exemplaireEmpruntesList.clear();
            exemplaireEmpruntesParUsagerList.clear();

            // Vider les données dans les TableView
            nonBorrowedTableView.getItems().clear();
            borrowedTableView.getItems().clear();
            currentUserBorrowedTableView.getItems().clear();

            // Recharger les nouvelles données depuis la base
            loadExemplaireData();
            nonBorrowedTableView.refresh();
            borrowedTableView.refresh();
            currentUserBorrowedTableView.refresh();

            setupSearchFilter();


            afficherMessage("Succès", "Livre retourné avec succès !");
        } else {
            afficherMessage("Erreur", "Veuillez sélectionner un livre à retourner.");
        }
    }

    // Charger les exemplaires dans les différentes tables
    private void loadExemplaireData() {
        List<Exemplaire> exemplaires = exemplaireDataLoader.getAllExemplairesWithDetails();

        // Filtrer les exemplaires non empruntés et empruntés
        for (Exemplaire exemplaire : exemplaires) {
            if (exemplaire.getEmprunte()) {
                exemplaireEmpruntesList.add(exemplaire);

            } else {
                exemplaireNonEmpruntesList.add(exemplaire);
            }
        }

        // Charger les exemplaires empruntés par l'usager actuel
        Usager usager = Session.getUsagerConnecte();
        for (Exemplaire exemplaire : exemplaireEmpruntesList) {
            if (reservationService.isExemplaireBorrowedByUser(exemplaire, usager)) {
                exemplaireEmpruntesParUsagerList.add(exemplaire);
            }
        }
        // Teste après récupération des données
        exemplaires.forEach(exemplaire -> {
            Oeuvre oeuvre = exemplaire.getEdition().getOeuvre();
        });
        // Assigner les données aux TableView
        nonBorrowedTableView.setItems(exemplaireNonEmpruntesList);
        borrowedTableView.setItems(exemplaireEmpruntesList);
        currentUserBorrowedTableView.setItems(exemplaireEmpruntesParUsagerList);
    }

    // Méthode pour afficher des messages d'alerte
    private void afficherMessage(String titre, String contenu) {
        Alert alert = new Alert(Alert.AlertType.INFORMATION);
        alert.setTitle(titre);
        alert.setContentText(contenu);
        alert.showAndWait();
    }

    // Méthode de déconnexion
    @FXML
    private void logout() {
        Session.setUsagerConnecte(null);  // Réinitialiser la session
        System.out.println("Déconnexion réussie.");
        // Fermer la fenêtre actuelle
        Stage stage = (Stage) btnLogout1.getScene().getWindow(); // Récupérer le stage actuel à partir d'un élément de la scène (comme un bouton)
        stage.close();

        // Fermer la fenêtre actuelle et retourner à l'écran de connexion
        HelloApplication.showLoginWindow();
    }
}
