package com.example.biblio;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;

import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.util.Objects;

public class LoginController {

    @FXML
    private GridPane loginPane;

    @FXML
    private TextField emailField;

    @FXML
    private PasswordField passwordField;

    private final UsagerService userConnection;

    private HelloApplication mainApp; // Référence à l'application principale
    private Stage stage; // Référence au stage


    @FXML
    public void initialize() {
        // Associer l'événement "KeyPressed" à toute la fenêtre
        loginPane.setOnKeyPressed(this::handleKeyPress);
    }

    // Gestion de l'appui sur la touche "Enter"
    private void handleKeyPress(javafx.scene.input.KeyEvent event) {
        if (event.getCode() == KeyCode.ENTER) {
            handleLogin();  // Appelle la méthode de connexion quand Enter est pressé
        }
    }


    public LoginController() {
        this.userConnection = new UsagerService(); // Le service utilisateur pour gérer la vérification
    }

    public void setMainApp(HelloApplication mainApp) {
        this.mainApp = mainApp;
    }

    public void setStage(Stage stage) {
        this.stage = stage;
    }

    @FXML
    public void handleLogin() {
        String adresseEmail = emailField.getText();
        String password = passwordField.getText();


        if (adresseEmail.isEmpty() || password.isEmpty()) {
            showAlert(AlertType.ERROR, "Erreur de connexion","Veuillez renseigner une adresse mail et un mot de passe");
        }

        else {
            Usager usager = userConnection.getUsagerByAdresseEmail(adresseEmail);

            if (usager == null) {
                showAlert(Alert.AlertType.ERROR, "Erreur", "L'adresse email n'est pas enregistrée. Veuillez vous créer un compte.");
                return;
            }

            // Vérification des identifiants avec la méthode de UserService
            if (userConnection.checkPassword(password, usager)) {
                if (usager != null) {
                    if (usager.getCategorieUsagerId() == 3) {
                        Session.setUsagerConnecte(usager);
                        redirectToGestionView();

                    }else{
                    Session.setUsagerConnecte(usager);  // Sauvegarde dans la session
                    redirectToMainView();}
                }
            } else {
                showAlert(AlertType.ERROR, "Erreur de connexion", "Adresse mail ou mot de passe incorrect.");
            }
        }
    }

    private void redirectToGestionView() {
        try {
            FXMLLoader fxmlloader = new FXMLLoader(getClass().getResource("/com/example/biblio/gestionnaire.fxml"));
            Parent root = fxmlloader.load();
            // Création d'un nouveau stage pour la scène principale
            Stage newStage = new Stage();
            newStage.setScene(new Scene(root));
            newStage.setMinWidth(1000);
            newStage.setMinHeight(800);
            newStage.setWidth(1000);
            newStage.setHeight(800);
            newStage.setTitle("Gestion de la Bibliothèque");
            newStage.show(); // Affiche la nouvelle fenêtre
            Stage stage = (Stage) passwordField.getScene().getWindow();
            stage.close();

        } catch (IOException e) {
            System.err.println("Erreur lors du chargement de la scène principale : " + e.getMessage());
        }
    }

    private void redirectToMainView() {
        try {
            loadMainScene(); // Charge la scène principale
        } catch (IOException e) {
            System.err.println("Erreur lors du chargement de la scène principale : " + e.getMessage());
        }
    }

    public void loadMainScene() throws IOException {
        FXMLLoader fxmlloader = new FXMLLoader(getClass().getResource("/com/example/biblio/main.fxml"));
        Parent root = fxmlloader.load();
        Stage newStage = new Stage(); // Création d'un nouveau stage pour la scène principale
        newStage.setScene(new Scene(root));
        newStage.setMinWidth(1000);
        newStage.setMinHeight(800);
        newStage.setWidth(1000);
        newStage.setHeight(800);
        newStage.setTitle("Bibliothèque en Ligne");
        newStage.show(); // Affiche la nouvelle fenêtre

        // Fermer la fenêtre de connexion après ouverture de la nouvelle scène
        if (this.stage != null) {
            this.stage.close();
        }
    }

    @FXML
    public void handleCreateAccount() {
        try {
            loadCreateAccountScene(); // Charger la scène de création de compte
        } catch (IOException e) {
            System.err.println("Erreur lors du chargement de la scène de création de compte : " + e.getMessage());
        }
    }

    public void loadCreateAccountScene() throws IOException {
        FXMLLoader fxmlloader = new FXMLLoader(getClass().getResource("/com/example/biblio/register.fxml"));
        Parent root = fxmlloader.load();
        Stage newStage = new Stage(); // Création d'un nouveau stage pour la scène principale
        newStage.setScene(new Scene(root));
        newStage.setMinWidth(1000);
        newStage.setMinHeight(800);
        newStage.setWidth(1000);
        newStage.setHeight(800);
        newStage.setTitle("Créer un Compte");
        newStage.show(); // Affiche la nouvelle fenêtre

        Stage stage = (Stage) passwordField.getScene().getWindow();
        stage.close();
    }

    private void showAlert(AlertType alertType, String title, String message) {
        Alert alert = new Alert(alertType);
        alert.setTitle(title);
        alert.setHeaderText(null);
        alert.setContentText(message);
        alert.showAndWait();
    }
}
