package com.example.biblio;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;
import org.mindrot.jbcrypt.BCrypt;

import java.io.File;
import java.io.IOException;

public class SignUpController {

    @FXML
    private GridPane registerPane;

    @FXML
    private TextField prenomField;

    @FXML
    private TextField nomField;

    @FXML
    private TextField emailField;

    @FXML
    private PasswordField passwordField;

    @FXML
    private PasswordField confirmPasswordField;

    private UsagerService userService;

    private LoginController mainApp; // Référence à l'application principale
    private Stage stage; // Référence au stage

    @FXML
    public void initialize() {
        // Associer l'événement "KeyPressed" à toute la fenêtre
        registerPane.setOnKeyPressed(this::handleKeyPress);
    }

    private void handleKeyPress(KeyEvent keyEvent) {
        if (keyEvent.getCode() == KeyCode.ENTER) {
            handleSignUp();  // Appelle la méthode de connexion quand Enter est pressé
        }
    }

    public SignUpController() {
        this.userService = new UsagerService(); // Service pour gérer les utilisateurs
    }


    @FXML
    private void handleSignUp() {
        String prenom = prenomField.getText();
        String nom = nomField.getText();
        String email = emailField.getText();
        String password = passwordField.getText();
        String confirmPassword = confirmPasswordField.getText();

        // Vérification si les champs ne sont pas vides
        if (email.isEmpty() || password.isEmpty()) {
            showAlert(Alert.AlertType.ERROR, "Erreur", "L'adresse email et le mot de passe ne peuvent pas être vides.");
            return;
        }
        // Validation des champs
        if (!password.equals(confirmPassword)) {
            System.out.println("Les mots de passe ne correspondent pas.");
            return;
        }

        if (userService.getUsagerByAdresseEmail(email) != null) {
            showAlert(Alert.AlertType.ERROR, "Erreur", "Il y a déjà un compte existant avec cette adresse mail.");
            return;
        }

        try {
            // Hash du mot de passe
            String salt = BCrypt.gensalt(12);
            String hashedPassword = BCrypt.hashpw(password, salt);

            //Numéro d'identification obtenu en Hashant le mail :
            String numeroIdentification = IdentificationGenerator.genererNumeroIdentification(prenom, nom);

            // Créer un nouvel objet Usager
            Usager newUser = new Usager(prenom, nom, email, numeroIdentification, 1, hashedPassword, salt, false);

            // Ajouter l'usager à la base de données
            userService.addUsager(newUser);

            showAlert(Alert.AlertType.INFORMATION, "Succès", "Compte créé avec succès !");
            Session.setUsagerConnecte(newUser);
            redirectToMainView();

        } catch (Exception e) {
            showAlert(Alert.AlertType.ERROR, "Erreur", "Erreur lors de la création du compte : " + e.getMessage());
        }
    }

    private void redirectToMainView() {
        try {
            loadMainScene(); // Charge la scène principale
        } catch (IOException e) {
            System.err.println("Erreur lors du chargement de la scène principale : " + e.getMessage());
        }
    }

    public void loadMainScene() throws IOException {
        FXMLLoader fxmlloader = new FXMLLoader(getClass().getResource("/com/example/biblio/main.fxml"));

        Parent root = fxmlloader.load();
        Stage newStage = new Stage(); // Création d'un nouveau stage pour la scène principale
        newStage.setScene(new Scene(root));
        newStage.setMinWidth(1000);
        newStage.setMinHeight(800);
        newStage.setWidth(1000);
        newStage.setHeight(800);
        newStage.setTitle("Bibliothèque en Ligne");
        Stage stage = (Stage) prenomField.getScene().getWindow();
        stage.close();
        newStage.show(); // Affiche la nouvelle fenêtre
    }

    private void showAlert(Alert.AlertType alertType, String title, String message) {
        Alert alert = new Alert(alertType);
        alert.setTitle(title);
        alert.setHeaderText(null);
        alert.setContentText(message);
        alert.showAndWait();
    }
}
