package com.example.biblio;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import org.mindrot.jbcrypt.BCrypt;

public class UsagerService {
    private static DatabaseConnection databaseConnection;

    public UsagerService() {
        databaseConnection = new DatabaseConnection();
    }

    // Ajouter un usager
    public boolean addUsager(Usager usager) {
        String query = "INSERT INTO Usager (prenom, nom, adresseEmail, numeroIdentification, categorie_usager_id, Hashpassword, salt) VALUES (?, ?, ?, ?, ?, ?, ?)";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, usager.getPrenom());
            preparedStatement.setString(2, usager.getNom());
            preparedStatement.setString(3, usager.getAdresseEmail());
            preparedStatement.setString(4, usager.getNumeroIdentification());
            preparedStatement.setInt(5, usager.getCategorieUsagerId());
            preparedStatement.setString(6, usager.getPassword());
            preparedStatement.setString(7, usager.getSalt());
            preparedStatement.executeUpdate();
            System.out.println("Usager ajouté : " + usager.getPrenom() + " " + usager.getNom());
            return true;
        } catch (SQLException e) {
            System.err.println("Erreur lors de l'ajout de l'usager : " + e.getMessage());
            return false;
        }
    }

    // Obtenir un usager par AdresseMail
    public Usager getUsagerByAdresseEmail(String adresseEmail) {
        String query = "SELECT * FROM Usager WHERE adresseEmail = ?";
        Usager usager = null;

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, adresseEmail);
            ResultSet resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                usager = new Usager(
                        resultSet.getString("prenom"),
                        resultSet.getString("nom"),
                        resultSet.getString("adresseEmail"),
                        resultSet.getString("numeroIdentification"),
                        resultSet.getInt("categorie_usager_id"),
                        resultSet.getString("Hashpassword"),
                        resultSet.getString("salt"),
                        resultSet.getBoolean("surListeRouge")
                );
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération de l'usager : " + e.getMessage());
        }
        return usager;
    }

    // Obtenir un usager par ID
    public static Usager getUsagerById(int id) {
        String query = "SELECT * FROM Usager WHERE id = ?";
        Usager usager = null;

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, String.valueOf(id));
            ResultSet resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                usager = new Usager(
                        resultSet.getString("prenom"),
                        resultSet.getString("nom"),
                        resultSet.getString("adresseEmail"),
                        resultSet.getString("numeroIdentification"),
                        resultSet.getInt("categorie_usager_id"),
                        resultSet.getString("Hashpassword"),
                        resultSet.getString("salt"),
                        resultSet.getBoolean("surListeRouge")
                );
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération de l'usager : " + e.getMessage());
        }
        return usager;
    }

    // Obtenir tous les usagers
    public static List<Usager> getAllUsagers() {
        String query = "SELECT * FROM Usager";
        List<Usager> usagers = new ArrayList<>();

        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            while (resultSet.next()) {
                Usager usager = new Usager(
                        resultSet.getString("prenom"),
                        resultSet.getString("nom"),
                        resultSet.getString("adresseEmail"),
                        resultSet.getString("numeroIdentification"),
                        resultSet.getInt("categorie_usager_id"),
                        resultSet.getString("Hashpassword"),
                        resultSet.getString("salt"),
                        resultSet.getBoolean("surListeRouge")
                );
                usagers.add(usager);
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération des usagers : " + e.getMessage());
        }
        return usagers;
    }

    // Mettre à jour un usager
    public void updateUsager(Usager usager) {
        String query = "UPDATE Usager SET prenom = ?, nom = ?, adresseEmail = ?, categorie_usager_id = ? WHERE numeroIdentification = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, usager.getPrenom());
            preparedStatement.setString(2, usager.getNom());
            preparedStatement.setString(3, usager.getAdresseEmail());
            preparedStatement.setString(4, usager.getNumeroIdentification());
            preparedStatement.setInt(5, usager.getCategorieUsagerId());
            preparedStatement.executeUpdate();
            System.out.println("Usager mis à jour : " + usager.getPrenom() + " " + usager.getNom());
        } catch (SQLException e) {
            System.err.println("Erreur lors de la mise à jour de l'usager : " + e.getMessage());
        }
    }


    // Supprimer un usager
    public void deleteUsager(int id) {
        String query = "DELETE FROM Usager WHERE id = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, id);
            preparedStatement.executeUpdate();
            System.out.println("Usager supprimé avec l'ID : " + id);
        } catch (SQLException e) {
            System.err.println("Erreur lors de la suppression de l'usager : " + e.getMessage());
        }
    }

    public boolean checkPassword(String password, Usager user) {
        String salt = user.getSalt(); // Récupérer le sel depuis l'objet User
        String hashedPasswordFromDb = user.getPassword();

        // Recréer le hachage avec le même sel
        String hashedPassword = BCrypt.hashpw(password, salt);

        // Comparer avec le mot de passe haché stocké dans la base de données
        return hashedPassword.equals(hashedPasswordFromDb);
    }

    public int getIdFromEmail(String adresseEmail) {
        String query = "SELECT id FROM Usager WHERE adresseEmail = ?";

        Integer id = null;
        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setString(1, adresseEmail);
            ResultSet resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                id = resultSet.getInt("id");
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la récupération de l'usager : " + e.getMessage());
        }
        return id;
    }
}
